@@ -19,6 +19,9 @@
  */
 
 #include "config.h"
+#if defined (HAVE_SIGNAL_H)
+#include <signal.h>
+#endif
 #include "actions.h"
 #include "foreach.h"
 #include <gphoto2/gphoto2-port-info-list.h>
@@ -148,7 +151,7 @@ static int
 get_path_for_file (const char *folder, const char *name, CameraFileType type, CameraFile *file, char **path)
 {
 	unsigned int i, l;
-	char *s, b[1024];
+	char *s = NULL, b[1024];
 	time_t t = 0;
 	struct tm *tm;
 	int hour12;
@@ -358,6 +361,14 @@ get_path_for_file (const char *folder, const char *name, CameraFileType type, Ca
 		}
 	}
 
+	/**
+	 * If the file is a capture_preview,
+	 * apply prefix over the calculated basename
+	 */
+	if (type == GP_FILE_TYPE_PREVIEW) {
+		return gp_file_get_name_by_type (file, s, type, path);
+	}
+
 	return (GP_OK);
 }
 
@@ -437,7 +448,8 @@ save_camera_file_to_file (
 		unlink(s);
 		if (-1 == rename (curname, s)) {
 			/* happens if the user specified a absolute path with --filename */
-			if (errno == EXDEV) {
+			/* EPERM happens on windows, see https://github.com/gphoto/libgphoto2/issues/97 */
+			if ((errno == EXDEV) || (errno == EPERM)) {
 				char buf[8192];
 				int in_fd, out_fd;
 
@@ -457,6 +469,8 @@ save_camera_file_to_file (
 						break;
 					}
 				}
+				close(out_fd);
+				close(in_fd);
 				unlink(curname);
 			} else
 				perror("rename");
@@ -825,6 +839,14 @@ save_captured_file (CameraFilePath *path, int download) {
 				return (result);
 			}
 		}
+		if (	(gp_params.flags & FLAGS_KEEP_RAW) &&
+			(!strstr(path->name,".jpg") && !strstr(path->name,".JPG"))
+		) {
+			if (!(gp_params.flags & FLAGS_QUIET))
+				printf (_("Keeping file %s%s%s on the camera\n"),
+					path->folder, pathsep, path->name);
+			return GP_OK;
+		}
 
 		result = get_file_common (path->name, GP_FILE_TYPE_NORMAL);
 		if (result != GP_OK) {
@@ -1019,6 +1041,8 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 					(result == GP_ERROR_CANCEL)		||
 					(result == GP_ERROR_NO_SPACE)		||
 					(result == GP_ERROR_IO_USB_CLAIM)	||
+					(result == GP_ERROR_IO_LOCK)		||
+					(result == GP_ERROR_CAMERA_BUSY)	||
 					(result == GP_ERROR_OS_FAILURE)
 				)
 					return (result);
@@ -1083,7 +1107,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 				 */
 				do {
 					next_pic_time.tv_sec += glob_interval;
-				} while (timediff_now (&next_pic_time) < 0);
+				} while (glob_interval && (timediff_now (&next_pic_time) < 0));
 			}
 			capture_now = 0;
 		} else {
@@ -1265,9 +1289,9 @@ typedef enum {
 	ARG_CAPTURE_IMAGE_AND_DOWNLOAD,
 	ARG_CAPTURE_MOVIE,
 	ARG_CAPTURE_PREVIEW,
+	ARG_SHOW_PREVIEW,
 	ARG_CAPTURE_SOUND,
 	ARG_CAPTURE_TETHERED,
-	ARG_CAPTURE_TETHERED_KEEP,
 	ARG_CONFIG,
 	ARG_DEBUG,
 	ARG_DEBUG_LOGLEVEL,
@@ -1294,6 +1318,7 @@ typedef enum {
 	ARG_HELP,
 	ARG_HOOK_SCRIPT,
 	ARG_KEEP,
+	ARG_KEEP_RAW,
 	ARG_LIST_CAMERAS,
 	ARG_LIST_ALL_CONFIG,
 	ARG_LIST_CONFIG,
@@ -1436,6 +1461,10 @@ cb_arg_init (poptContext __unused__ ctx,
 	case ARG_NEW:
 		gp_params.flags |= FLAGS_NEW;
 		break;
+	case ARG_KEEP_RAW:
+		gp_params.flags |= FLAGS_KEEP_RAW;
+		break;
+
 	case ARG_KEEP:
 		gp_params.flags |= FLAGS_KEEP;
 		break;
@@ -1559,6 +1588,9 @@ cb_arg_run (poptContext __unused__ ctx,
 	case ARG_CAPTURE_PREVIEW:
 		params->p.r = action_camera_capture_preview (&gp_params);
 		break;
+	case ARG_SHOW_PREVIEW:
+		params->p.r = action_camera_show_preview (&gp_params);
+		break;
 	case ARG_CAPTURE_SOUND:
 		params->p.r = capture_generic (GP_CAPTURE_SOUND, arg, 0);
 		break;
@@ -1638,6 +1670,12 @@ cb_arg_run (poptContext __unused__ ctx,
 		GPPort		*port;
 		GPPortInfo	info;
 
+		/* If a camera is already open, close it, as we need a new port */
+		if (gp_params.camera) {
+			gp_camera_exit (gp_params.camera, gp_params.context);
+			/* exit, not free. will reopen on next command. */
+		}
+
 		params->p.r = gp_port_new (&port);
 		if (params->p.r != GP_OK) {
 			gp_log(GP_LOG_ERROR,"port_reset", "new failed %d", params->p.r);
@@ -1939,7 +1977,7 @@ main (int argc, char **argv, char **envp)
 		 N_("Set debug level [error|debug|data|all]"), NULL},
 		{"debug-logfile", '\0', POPT_ARG_STRING, (void *) &debug_logfile_name, ARG_DEBUG_LOGFILE,
 		 N_("Name of file to write debug info to"), N_("FILENAME")},
-		{"quiet", '\0', POPT_ARG_NONE, NULL, ARG_QUIET,
+		{"quiet", 'q', POPT_ARG_NONE, NULL, ARG_QUIET,
 		 N_("Quiet output (default=verbose)"), NULL},
 		{"hook-script", '\0', POPT_ARG_STRING, NULL, ARG_HOOK_SCRIPT,
 		 N_("Hook script to call after downloads, captures, etc."),
@@ -1967,7 +2005,7 @@ main (int argc, char **argv, char **envp)
 		{"list-ports", '\0', POPT_ARG_NONE, NULL, ARG_LIST_PORTS,
 		 N_("List supported port devices"), NULL},
 		{"abilities", 'a', POPT_ARG_NONE, NULL, ARG_ABILITIES,
-		 N_("Display camera/driver abilities"), NULL},
+		 N_("Display the camera/driver abilities in the libgphoto2 database"), NULL},
 		POPT_TABLEEND
 	};
 	const struct poptOption configOptions[] = {
@@ -1996,15 +2034,20 @@ main (int argc, char **argv, char **envp)
 		GPHOTO2_POPT_CALLBACK
 		{"keep", '\0', POPT_ARG_NONE, NULL, ARG_KEEP,
 		 N_("Keep images on camera after capturing"), NULL},
+		{"keep-raw", '\0', POPT_ARG_NONE, NULL, ARG_KEEP_RAW,
+		 N_("Keep RAW images on camera after capturing"), NULL},
 		{"no-keep", '\0', POPT_ARG_NONE, NULL, ARG_NO_KEEP,
 		 N_("Remove images from camera after capturing"), NULL},
 		{"wait-event", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL, ARG_WAIT_EVENT,
-		 N_("Wait for event(s) from camera"), N_("COUNT")},
+		 N_("Wait for event(s) from camera"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		{"wait-event-and-download", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
-		 ARG_CAPTURE_TETHERED, N_("Wait for event(s) from the camera and download new images"), N_("COUNT")},
+		 ARG_CAPTURE_TETHERED, N_("Wait for event(s) from the camera and download new images"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		{"capture-preview", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_PREVIEW,
 		 N_("Capture a quick preview"), NULL},
+		{"show-preview", '\0', POPT_ARG_NONE, NULL,
+		 ARG_SHOW_PREVIEW,
+		 N_("Show a quick preview as Ascii Art"), NULL},
 		{"bulb", 'B', POPT_ARG_INT, NULL, ARG_CAPTURE_BULB,
 		 N_("Set bulb exposure time in seconds"), N_("SECONDS")},
 		{"frames", 'F', POPT_ARG_INT, NULL, ARG_CAPTURE_FRAMES,
@@ -2024,9 +2067,7 @@ main (int argc, char **argv, char **envp)
 		{"capture-sound", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_SOUND, N_("Capture an audio clip"), NULL},
 		{"capture-tethered", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
-		 ARG_CAPTURE_TETHERED, N_("Wait for shutter release on the camera and download"), N_("COUNT")},
-		{"trigger-capture", '\0', POPT_ARG_NONE, NULL,
-		 ARG_TRIGGER_CAPTURE, N_("Trigger image capture"), NULL},
+		 ARG_CAPTURE_TETHERED, N_("Wait for shutter release on the camera and download"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		POPT_TABLEEND
 	};
 	const struct poptOption fileOptions[] = {
@@ -2101,12 +2142,12 @@ main (int argc, char **argv, char **envp)
 
 #ifdef HAVE_LIBEXIF
 		{"show-exif", '\0', POPT_ARG_STRING, NULL, ARG_SHOW_EXIF,
-		 N_("Show EXIF information"), NULL},
+		 N_("Show EXIF information of JPEG images"), NULL},
 #endif
 		{"show-info", '\0', POPT_ARG_STRING, NULL, ARG_SHOW_INFO,
-		 N_("Show info"), NULL},
+		 N_("Show image information, like width, height, and capture time"), NULL},
 		{"summary", '\0', POPT_ARG_NONE, NULL, ARG_SUMMARY,
-		 N_("Show summary"), NULL},
+		 N_("Show camera summary"), NULL},
 		{"manual", '\0', POPT_ARG_NONE, NULL, ARG_MANUAL,
 		 N_("Show camera driver manual"), NULL},
 		{"about", '\0', POPT_ARG_NONE, NULL, ARG_ABOUT,
@@ -2254,9 +2295,9 @@ main (int argc, char **argv, char **envp)
 	CHECK_OPT (ARG_CAPTURE_IMAGE_AND_DOWNLOAD);
 	CHECK_OPT (ARG_CAPTURE_MOVIE);
 	CHECK_OPT (ARG_CAPTURE_PREVIEW);
+	CHECK_OPT (ARG_SHOW_PREVIEW);
 	CHECK_OPT (ARG_CAPTURE_SOUND);
 	CHECK_OPT (ARG_CAPTURE_TETHERED);
-	CHECK_OPT (ARG_CAPTURE_TETHERED_KEEP);
 	CHECK_OPT (ARG_CONFIG);
 	CHECK_OPT (ARG_DELETE_ALL_FILES);
 	CHECK_OPT (ARG_DELETE_FILE);
@@ -2444,6 +2485,7 @@ main (int argc, char **argv, char **envp)
 	}
 
         signal (SIGINT, signal_exit);
+        signal (SIGTERM, signal_exit);
 
 	/* If we are told to be quiet, be so. *
 	cb_params.type = CALLBACK_PARAMS_TYPE_QUERY;
